@@ -1,84 +1,77 @@
 #' Joint Random Forest for the simultaneous estimation of multiple related networks
-#' (This file is a modified version of file RF.R contained in the R package randomForest.)
 #'
-#' @param x numeric matrix ((C by r) by n): rows correspond to predictor variables for each class and columns correspond to the maximum number of samples over classes. Missing values are not allowed. 
-#' @param y numeric matrix (C by n): rows correspond to response variables for each class and columns correspond to the maximum number of samples over classes. Missing values are not allowed. 
-#' @param xtest a data frame or matrix (like x) containing predictors for the test set. 
-#' @param ytest response for the test set. 
-#' @param ntree numeric value: number of trees. 
-#' @param sampsize numeric vector (C by 1): number of samples for each class of data. 
-#' @param totsize Max number of samples across different classes 
-#' @param mtry numeric value: number of predictors to be sampled at each node. 
-#' @param replace Should sampling of cases be done with or without replacement? 
-#' @param classwt Priors of the classes.  Need not add up to one. Ignored for regression. 
-#' @param cutoff (Classification only)  A vector of length equal to number of classes.  The winning class for an observation is the one with the maximum ratio of proportion of votes to cutoff. Default is 1/k where k is the number of classes  
-#' @param strata A (factor) variable that is used for stratified sampling. 
-#' @param nodesize Minimum size of terminal nodes.  Setting this number larger causes smaller trees to be grown (and thus take less time). Note that the default values are different for classification (1) and regression (5). 
-#' @param maxnodes Maximum number of terminal nodes trees in the forest can have.  If not given, trees are grown to the maximum possible  (subject to limits by nodesize ).  If set larger than maximum possible, a warning is issued. 
-#' @param importance Should importance of predictors be assessed?  
-#' @param localImp Should casewise importance measure be computed? 
-#' @param nPerm Number of times the OOB data are permuted per tree for assessing variable importance.  Number larger than 1 gives slightly more stable estimate, but not very effectiv. Currently only implemented for regression. 
-#' @param proximity Should proximity measure among the rows be calculated? 
-#' @param oob.prox Should proximity be calculated only on out-of-bag data? 
-#' @param norm.votes If TRUE (default), the final result of votes are expressed as fractions. If FALSE, raw vote counts are returned (useful for combining results from different runs). Ignored for regression. 
-#' @param do.trace If set to TRUE, give a more verbose output as randomForest is run. If set to some integer, then running output is printed for every do.trace trees. 
-#' @param keep.forest If set to FALSE, the forest will not be retained in the output object. If xtest is given, defaults to FALSE. 
-#' @param corr.bias perform bias correction for regression? Note: Experimental. Use at your own risk. 
-#' @param keep.inbag Should an n by ntree matrix be returned that keeps track of which samples are in-bag in which trees (but not how many times, if sampling with replacement) 
-#' @param nclasses numeric value: the total number of classes C. 
-#' @param ... optional parameters to be passed to the low level function 
+#' MAIN FUNCTION -- > JRF
 #' 
-#' @examples
+#' INPUT
 #' 
-#' # --- Derive weighted networks via JRF
+#' X            list object containing data for each class
+#' ntree        number of trees
+#' mtry         number of variables to be sampled at each node
+#' genes.name   list of gene names 
 #' 
-#' nclasses=2               # number of data sets / classes
-#' n1<-n2<-50               # sample size for each data sets
-#' p<-100                   # number of variables (genes)
-#' 
-#'   # --- Generate data sets
-#' 
-#' data1<-matrix(rnorm(p*n1),p,n1)       # generate data1
-#' data2<-matrix(rnorm(p*n2),p,n1)       # generate data2
-#' 
-#'   # --- Standardize variables to mean 0 and variance 1
-#'   
-#'  data1 <- t(apply(data1, 1, function(x) { (x - mean(x)) / sd(x) } ))
-#'  data2 <- t(apply(data2, 1, function(x) { (x - mean(x)) / sd(x) } ))
+#' OUTPUT: importance score of interactions
 #'
-#'   # --- Initialize variables
-#'   
-#'  imp1<-imp2<-matrix(0,p,p)   # matrix to store importance scores
-#'  ntree=1000;                 # number of trees
-#'  nsample<-c(n1,n2)           # vector containing sample size for each class
-#' 
-#' # --- run JRF for each target gene
-#' for (j in 1:2){   # for loop over target genes
-#' 
-#'   y<-matrix(0,2,max(n1,n2));  #--- create matrix (classes by n=max(n1,n2)) of response variable (expression of target gene j)
-#'   y[1,seq(1,n1)]<-as.matrix(data1[j,])     
-#'   y[2,seq(1,n2)]<-as.matrix(data2[j,])
-#'   
-#'   x<-matrix(0,p*2-2,max(n1,n2)) #--- create matrix of covariates (expression of genes)
-#'   x[seq(1,p-1),seq(1,n1)]<-as.matrix(data1[-j,])
-#'   x[seq(p,2*p-2),seq(1,n2)]<-as.matrix(data2[-j,])    
-#'   
-#'    jrf.out<-JRF(x=x,y=y,mtry=round(sqrt(p-1)),importance=TRUE,sampsize=nsample,nclasses=nclasses,ntree=ntree)
-#'      
-#'    imp1[-j,j]<-importance(jrf.out,scale=FALSE)[seq(1,p-1)]      #- save importance score for net1
-#'    imp2[-j,j]<-importance(jrf.out,scale=FALSE)[seq(p,(p-1)*2)]  #- save importance score for net2
-#'    
-#' }
-#' 
-#' @useDynLib JRF, regRF, .registration = TRUE
-#' @return out object of class JRF
 #'
-#' @export 
-
-#"JRF" <-  function(x, ...)UseMethod("JRF")
+#' OTHER FUNCTIONS -- > importance  and  JRF_onetarget
+#' 
+#' importance     compute importance score for an object of class JRF 
+#' (this file is a modified version of file importance contained in package randomForest, A. Liaw and M. Wiener (2002))
+#' 
+#' JRF_onetarget  for each class, model the expression of a target gene as a function of the expression of other genes via random forest. 
+#'                class specific tree ensemble are designed to borrow information across them. 
+#' (this file is a modified version of file randomForest contained in package randomForest, A. Liaw and M. Wiener (2002))
+#'   
+#"JRF" <-  function(X, ...)UseMethod("JRF")
 
 
-"JRF" <-
+importance <- function(x,  scale=TRUE) {
+  # --- Function importance is a modified version of function importance from R package randomForest
+  
+  type=NULL;
+  class=NULL;
+  if (!inherits(x, "randomForest"))
+        stop("x is not of class randomForest")
+    classRF <- x$type != "regression"
+    hasImp <- !is.null(dim(x$importance)) || ncol(x$importance) == 1
+    hasType <- !is.null(type)
+    if (hasType && type == 1 && !hasImp)
+        stop("That measure has not been computed")
+    allImp <- is.null(type) && hasImp
+    if (hasType) {
+        if (!(type %in% 1:2)) stop("Wrong type specified")
+        if (type == 2 && !is.null(class))
+            stop("No class-specific measure for that type")
+    }
+    
+    imp <- x$importance
+    if (hasType && type == 2) {
+        if (hasImp) imp <- imp[, ncol(imp), drop=FALSE]
+    } else {
+        if (scale) {
+            SD <- x$importanceSD
+            imp[, -ncol(imp)] <-
+                imp[, -ncol(imp), drop=FALSE] /
+                    ifelse(SD < .Machine$double.eps, 1, SD)
+        }
+        if (!allImp) {
+            if (is.null(class)) {
+                ## The average decrease in accuracy measure:
+                imp <- imp[, ncol(imp) - 1, drop=FALSE]
+            } else {
+                whichCol <- if (classRF) match(class, colnames(imp)) else 1
+                if (is.na(whichCol)) stop(paste("Class", class, "not found."))
+                imp <- imp[, whichCol, drop=FALSE]
+            }
+        }
+    }
+    imp<-imp[,2]
+    imp
+}
+
+
+# --- Function JRF_onetarget is a modified version of function randomForest from R package randomForest
+
+"JRF_onetarget" <-
   function(x, y=NULL,  xtest=NULL, ytest=NULL, ntree,
            sampsize,              
            totsize = if (replace) ncol(x) else ceiling(.632*ncol(x)),
@@ -106,8 +99,6 @@
     n <- ncol(y)           # number of samples
     p <- nrow(x)/nclasses  # number of variables
     
-    
-    
     if (n == 0) stop("data (x) has 0 rows")
     x.row.names <- rownames(x)
     x.col.names <- if (is.null(colnames(x))) 1:ncol(x) else colnames(x)
@@ -474,3 +465,53 @@
     
   }
 
+
+# --- MAIN function
+"JRF" <-
+  function(X, ntree,mtry,genes.name) {
+
+    nclasses<-length(X)
+    sampsize<-rep(0,nclasses)
+    
+    for (j in 1:nclasses) sampsize[j]<-dim(X[[j]])[2]
+    
+    totsize<-tot<-max(sampsize);
+    p<-dim(X[[1]])[1];
+    imp<-array(0,c(p,length(genes.name),nclasses))
+    
+    imp.final<-matrix(0,p*(p-1)/2,nclasses);
+    vec1<-matrix(rep(genes.name,p),p,p)
+    vec2<-t(vec1)
+    vec1<-vec1[lower.tri(vec1,diag=FALSE)]
+    vec2<-vec2[lower.tri(vec2,diag=FALSE)]
+    
+    index<-seq(1,p)
+    
+    for (j in 1:length(genes.name)){
+
+      covar<-matrix(0,(p-1)*nclasses,tot)             
+      y<-matrix(0,nclasses,tot)             
+      
+      for (c in 1:nclasses)  {
+        y[c,seq(1,sampsize[c])]<-as.matrix(X[[c]][j,])
+        covar[seq((c-1)*(p-1)+1,c*(p-1)),seq(1,sampsize[c])]<-X[[c]][-j,]
+      }
+      
+      jrf.out<-JRF_onetarget(x=covar,y=y,mtry=mtry,importance=TRUE,sampsize=sampsize,nclasses=nclasses,ntree=ntree)
+      
+      for (s in 1:nclasses) imp[-j,j,s]<-importance(jrf.out,scale=FALSE)[seq((p-1)*(s-1)+1,(p-1)*(s-1)+p-1)]  #- save importance score for net1
+      
+      }
+      
+    # --- Derive importance score for each interaction 
+      for (s in 1:nclasses){ 
+         imp.s<-imp[,,s]; t.imp<-t(imp.s)
+         imp.final[,s]<-(imp.s[lower.tri(imp.s,diag=FALSE)]+t.imp[lower.tri(t.imp,diag=FALSE)])/2        
+      }
+    
+    out<-cbind(as.character(vec1),as.character(vec2),as.data.frame(imp.final),stringsAsFactors=FALSE)
+    colnames(out)<-c(paste0('gene',1:2),paste0('importance',1:nclasses))
+    return(out)
+    
+  }
+
